@@ -1,4 +1,4 @@
-/* $Id: e2p_mvbar.c 1082 2008-08-11 23:30:42Z tpgww $
+/* $Id: e2p_mvbar.c 1344 2008-11-02 06:53:19Z tpgww $
 **************************************
 
 Emelmovebar - a plugin for emelFM2
@@ -381,7 +381,7 @@ _e2p_mvbar_exec (VPATH *slocal, VPATH *dlocal, gboolean realmove,
 				}
 			}
 			else
-				usleep (1000);	//nominal only (cancellation point)
+				usleep (100000);	//no need to hog CPU by refreshing too fast (cancellation point)
 		}
 
 		//show the full-time score ASAP
