@@ -1,6 +1,6 @@
-/* $Id: e2p_rename.c 1244 2008-09-14 02:01:16Z tpgww $
+/* $Id: e2p_rename.c 1412 2009-02-01 21:51:22Z tpgww $
 
-Copyright (C) 2005-2008 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -555,7 +555,7 @@ static E2_TwResult _e2p_ren_twcb (VPATH *localpath,
 
 			if (_e2p_ren_match_name (s, twdata))
 			{
-				utf = D_FILENAME_FROM_LOCALE ((gchar *) localpath);	//always get a copy
+				utf = D_FILENAME_FROM_LOCALE (VPSTR(localpath));	//always get a copy
 				g_ptr_array_add (twdata->candidates, (gpointer) utf);
 			}
 			break;
@@ -931,6 +931,7 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 
 		local_btn = E2_BUTTON_YES;
 		local_btn.label = _("_Rename");
+		local_btn.stock = GTK_STOCK_CONVERT;
 		local_btn.showflags &= ~E2_BTN_DEFAULT;
 		e2_dialog_add_defined_button (dialog, &local_btn);
 
@@ -968,7 +969,7 @@ static void _e2p_ren_rename (E2_RenDialogRuntime *rt)
 			g_free (prompt);
 			gtk_widget_show (dialog);
 			*rt->status = E2_TASK_PAUSED;
-			e2_dialog_wait (dialog, TRUE, TRUE, multi, TRUE);
+			choice = e2_dialog_wait (dialog, TRUE, TRUE, multi, TRUE);
 			*rt->status = E2_TASK_RUNNING;
 			gtk_widget_hide (dialog);
 			gtk_widget_grab_focus (rt->dialog);
@@ -1638,11 +1639,14 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	_e2p_ren_create_radio_grouped_button (hbox, radio, _("Match regular e_xpression"),
 		OLD_REGEX_P, &rt);
 
-	hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, E2_PADDING);
-	e2_widget_add_mid_label (hbox, _("Current name is like this:"), 0.05, TRUE, 0);
-
+	hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, 0);
+	GtkWidget *wid =
+	e2_widget_add_mid_label (hbox, _("Current name is like this:"), 0.0, FALSE, E2_PADDING);
+	GtkSizeGroup *same = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	gtk_size_group_add_widget (same, wid);
 	gdk_threads_enter ();
-	rt.pattern = e2_combobox_add (hbox, FALSE, E2_PADDING_XSMALL,
+
+	rt.pattern = e2_combobox_add (hbox, TRUE, E2_PADDING_XSMALL,
 		_e2p_ren_activation_cb, &rt, &pattern_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
 	gtk_entry_set_text (GTK_ENTRY (GTK_BIN (rt.pattern)->child), "(.*)");
@@ -1665,10 +1669,12 @@ static gboolean _e2p_renameQ (E2_ActionTaskData *qed)
 	_e2p_ren_create_toggle_grouped_button (hbox, radio,
 		_("New name is _lower case"), NEW_LOWER_P, &rt);
 	hbox = e2_widget_add_box (dialog_vbox, TRUE, 0, FALSE, FALSE, E2_PADDING);
-	_e2p_ren_create_toggle_button (hbox, _("_New name is like this:"),
+	wid = _e2p_ren_create_toggle_button (hbox, _("_New name is like this:"),
 		NEW_THIS_P, &rt);
 	gdk_threads_enter ();
-	rt.newpattern = e2_combobox_add (hbox, FALSE, E2_PADDING_XSMALL,
+	gtk_size_group_add_widget (same, wid);
+	g_object_unref (G_OBJECT (same));
+	rt.newpattern = e2_combobox_add (hbox, TRUE, E2_PADDING_XSMALL,
 		_e2p_ren_activation_cb, &rt, &newpattern_history,
 		E2_COMBOBOX_HAS_ENTRY | E2_COMBOBOX_FOCUS_ON_CHANGE);
 	state = _e2p_ren_get_flag (NEW_THIS_P);
