@@ -1,4 +1,4 @@
-/* $Id: e2p_unpack.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2p_unpack.c 1367 2008-12-31 09:18:19Z tpgww $
 
 Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
@@ -356,7 +356,7 @@ install/uninstall scripts, the other the files to be installed.
 	F_FREE (ddata.locapath);
 #else
 	gchar *local = F_FILENAME_TO_LOCALE (package);
-	gint index = _e2p_unpack_match_type (local);
+	gint index = _e2p_unpack_match_type ((VPATH*)local);
 	F_FREE (local);
 #endif
 	if (index == -1)
@@ -539,7 +539,7 @@ decompress UNICODE: 7za x archive.7z -odirname -aoa
 	g_free (ddata.localpath);
 #else
 	gchar *local = e2_utils_dircat (curr_view, info->filename, TRUE);
-	gint index = _e2p_unpack_match_type (local);
+	gint index = _e2p_unpack_match_type ((VPATH*)local);
 	g_free (local);
 #endif
 	if (index == -1)
