@@ -1,4 +1,4 @@
-/* $Id: e2_command.c 1333 2008-10-22 23:56:34Z tpgww $
+/* $Id: e2_command.c 1374 2009-01-13 04:43:42Z tpgww $
 
 Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
@@ -269,7 +269,7 @@ static void _e2_command_display (gchar *buffer, gboolean error, E2_TaskRuntime *
 				rsep = strchr (line, '\r');
 				if (rsep != NULL)
 				{
-					gchar rsave;
+					gchar rsave = 0;	//assignment for warning prevention only
 					if (rsep != nsep - 1)	//this is not a \r\n sequence
 					{
 						rsep++;
@@ -471,6 +471,9 @@ Assumes BGL is off
 */
 static gpointer _e2_command_watch (E2_TaskRuntime *rt)
 {
+#ifdef E2CMDTYPEB
+	gboolean outread, errread;
+#endif
 	struct timespec wait;
 	struct timespec *waitptr;
 	gchar *buffer;
@@ -495,11 +498,10 @@ static gpointer _e2_command_watch (E2_TaskRuntime *rt)
 	maxfd++;
 
 	//as the pipes will be selected at least when the child closes it's ok to
-	//block until select succeeds
+	//block until select succeeds NO -  BAD to block when a command ends immediately
 	//and note, ouput pipe (if used) is always selectable
-	waitptr = NULL;
+	waitptr = &wait; //NULL;
 #ifdef E2CMDTYPEB
-	gboolean outread, errread;
 	outread = errread = TRUE;	//we want to pselect() the fd's at least once
 #endif
 
@@ -508,11 +510,14 @@ static gpointer _e2_command_watch (E2_TaskRuntime *rt)
 		gint selcount;
 		guint maxwait;
 		ssize_t bytecount;
+#ifndef E2CMDTYPEB
 		gboolean outread, errread;
+#endif
 		fd_set in_fds;
 #ifdef OUTPUT_POLL
 		fd_set out_fds;
 #endif
+		maxwait = 1;	//initial poll-timeout is 1 sec.
 reselect:
 /*#ifdef E2CMDTYPEB
 		//FIXME handle case where outread and errread are both FALSE;
@@ -523,7 +528,7 @@ reselect:
 		}
 #endif */
 		FD_ZERO (&in_fds);
-//FIXME find a way to may these tests work without stuffing up the completion reporting
+//FIXME find a way to make these tests work without stuffing up the completion reporting
 #ifdef E2CMDTYPEB
 		if (outread)	//don't select fd's that are closed
 #endif
@@ -570,30 +575,40 @@ reselect:
 		else if (selcount > 0)
 		{
 //			printd (DEBUG, "pipe selection");
-			//at end-of-command, both input fd's report 0 bytes ready to read
+			//at end-of-command, both input fd's report EOF (0 bytes ready to read)
 			if (FD_ISSET (cmddata->child_stdout_fd, &in_fds))
 			{
 //				printd (DEBUG, "child stdout ready");
 				//read input, starting with normal pipe
-#ifdef E2CMDTYPEB
+#ifndef E2CMDTYPEB
 				outread = FALSE;	//in case this one fails
 #endif
 				while ((bytecount =
 					_e2_command_piperead (cmddata->child_stdout_fd, buffer, READMAX)) > 0)
 				{
 					printd (DEBUG, "child stdout buffer read");
+#ifndef E2CMDTYPEB
 					outread = TRUE;
+#endif
 					*(buffer+bytecount) = '\0';
 					_e2_command_display (buffer, FALSE, rt);
 					if (bytecount != READMAX)
 						break;
 				}
-				if (!outread)
+#ifdef E2CMDTYPEB
+				outread = (bytecount != 0);	//finished when 0 bytes read
+#endif
+#ifdef DEBUG_MESSAGES
+				if (bytecount == 0)		//EOF
+				{
+					printd (DEBUG, "child stdout EOF");
+				}
+				else if (!outread)
 				{
 					//pipe closed or closing
-					printd (DEBUG, "FALSE positive child stdout ready");
-					usleep (10000);	//allow for some cleanup
+					printd (DEBUG, "actually, child stdout pipe NOT ready: %s", strerror(errno));
 				}
+#endif
 			}
 //#ifdef E2CMDTYPEB
 //			else
@@ -615,25 +630,35 @@ reselect:
 			{
 //				printd (DEBUG, "child stderr ready");
 				//read input, starting with error pipe
-#ifdef E2CMDTYPEB
+#ifndef E2CMDTYPEB
 				errread = FALSE;	//in case this one fails
 #endif
 				while ((bytecount =
 					_e2_command_piperead (cmddata->child_stderr_fd, buffer, READMAX)) > 0)
 				{
 					printd (DEBUG, "child stderr buffer read");
+#ifndef E2CMDTYPEB
 					errread = TRUE;
+#endif
 					*(buffer+bytecount) = '\0';
 					_e2_command_display (buffer, TRUE, rt);
 					if (bytecount != READMAX)
 						break;
 				}
-				if (!errread)
+#ifdef E2CMDTYPEB
+				errread = (bytecount != 0);	//finished when 0 bytes read
+#endif
+#ifdef DEBUG_MESSAGES
+				if (bytecount == 0)
+				{
+					printd (DEBUG, "child stderr EOF");
+				}
+				else if (!errread)
 				{
 					//pipe closed or closing
-					printd (DEBUG, "FALSE positive child stderr ready");
-					usleep (10000);	//allow for some cleanup
+					printd (DEBUG, "actually, child stderr NOT ready: %s", strerror(errno));
 				}
+#endif
 			}
 //#ifdef E2CMDTYPEB
 //			else
@@ -666,15 +691,19 @@ reselect:
 #endif
 		}
 
+		if (!outread || !errread)
+			usleep (10000);	//allow for some cleanup
+
 		pthread_mutex_lock (&task_mutex);
 		gboolean done = (rt->status >= E2_TASK_COMPLETED);
 		pthread_mutex_unlock (&task_mutex);
-#ifdef E2CMDTYPEB
-		if (done || selcount == 0 || !(outread || errread))
-		{
-#else
-		if (done)
+		if (done
+//#ifdef E2CMDTYPEB
+//				|| selcount == 0 || !(outread || errread)
+//#endif
+			)
 		{
+#ifndef E2CMDTYPEB
 			if (selcount == 0 || !(outread || errread))
 			{
 #endif
@@ -933,7 +962,7 @@ static gint _e2_command_fork (gchar *command, gchar **args, const gchar *cwd,
 	}
 
 	e2_command_block_childsignal ();
-	gdk_threads_leave ();
+	gdk_threads_leave ();	//BGL management happens downstream
 
 	if ((pid = fork ()) == 0)
 	{	//child
@@ -1002,9 +1031,111 @@ static gint _e2_command_fork (gchar *command, gchar **args, const gchar *cwd,
 		_exit (1);
 		//end of child process
 	}
-	gdk_threads_enter ();
-	if (pid < 0)
-	{	//fork error
+	if (pid > 0)
+	{
+		//parent process
+		rt->status = E2_TASK_RUNNING;
+		rt->pid = (glong) pid;
+		rt->pidstr = g_strdup_printf ("%ld", rt->pid);
+		E2_CommandTaskData *cmddata = &rt->ex.command;
+		close (stdin_pipe[INWARDS]);
+		cmddata->child_stdin_fd = stdin_pipe[OUTWARDS];	//setup for later sending to child
+		cmddata->child_stdout_fd = stdout_pipe[INWARDS];	//setup for later receiving from child
+		close (stdout_pipe[OUTWARDS]);
+		cmddata->child_stderr_fd = stderr_pipe[INWARDS];	//setup for later receiving from child
+		close (stderr_pipe[OUTWARDS]);
+//		FILE *child_stdin_stream = fdopen (stdin_pipe[OUTWARDS], "w");
+//		FILE *child_stdout_stream = fdopen (stdout_pipe[INWARDS], "r");
+//		FILE *child_stderr_stream = fdopen (stderr_pipe[INWARDS], "r");
+//		cmddata->pollcount = 0;	//not ready to enable SIGCHILD yet NOT USED NOW
+		cmddata->sync = sync;
+		cmddata->extshell = extshell;
+		cmddata->show = show;
+#ifdef DEBUG_MESSAGES
+		if (sync)
+			printd (DEBUG, "synchronously run command %s", args[0]);
+		else
+			printd (DEBUG, "asynchronously run command %s", args[0]);
+#endif
+		if (show && e2_option_bool_get ("fileop-show"))
+		{
+			message = g_strconcat ((extshell) ? "sh>" : ">", command, NULL);
+			gdk_threads_enter ();
+			e2_output_print (rt->current_tab,
+				message, rt->pidstr, FALSE, "bold", "blue", NULL);
+			g_free (message);
+			message = g_strdup_printf ("  (%s)\n", rt->pidstr);
+			e2_output_print (rt->current_tab,
+				message, rt->pidstr, FALSE, "small", "blue", NULL);
+			gdk_threads_leave ();
+			g_free (message);
+//			WAIT_FOR_EVENTS_UNLOCKED;
+		}
+
+	//	e2_command_unblock_childsignal (); //allow child exit to be noticed SEE TIMER
+	/* seems to do nothing
+		Setting the O_ASYNC flag for the read end of a pipe causes a signal
+		(SIGIO by default) to be generated when new input becomes available on the pipe.
+		On Linux, O_ASYNC is supported for pipes only since kernel 2.6.
+
+		//make pipes non-blocking, so we can read big buffers without having
+		//to use fionread() to prevent blocking
+		gint flags = fcntl (stdout_pipe[INWARDS], F_GETFL);
+		if (flags != -1)
+			fcntl (stdout_pipe[INWARDS], F_SETFL, flags | O_NONBLOCK | O_ASYNC);
+		flags = fcntl (stderr_pipe[INWARDS], F_GETFL);
+		if (flags != -1)
+			fcntl (stderr_pipe[INWARDS], F_SETFL, flags | O_NONBLOCK | O_ASYNC);
+	*/
+	/*	prefer to poll pipes at least once before unblocking SIGCHILD in this thread
+		(ostensibly it's not valid to unblock SIGCHILD in another thread where watching
+		happens, but for some kernels that does seem to help, so unblocking also happens
+		in _e2_command_watch())
+		So for async at least we use a timer which checks a flag set in _e2_command_watch()
+		CHECKME can timer persist past rt destruction at session end ? etc ?
+		Timer interval is compromise between prompt cleanup when there is little or no
+		initial input, and the amount of interruption of the initial input loop when
+		that involves a lot of printing - so we use a 2-stage timer
+	*/
+		pthread_t wthreadID;
+		if (sync)
+		{
+			if (pthread_create (&wthreadID, NULL, (gpointer)_e2_command_watch, rt))
+			{
+				gdk_threads_enter (); //do this before unblocking ?
+				e2_command_unblock_childsignal ();
+			}
+			else
+			{
+//BAD HERE		gdk_threads_enter ();
+				e2_command_unblock_childsignal ();
+				//FIXME allow command output to be printed while waiting for completion
+				//probably by running as for async, but blocking start of any other action
+				// or command (except send to child)
+				pthread_join (wthreadID, NULL);
+				gdk_threads_enter ();
+				return cmddata->exit;
+			}
+		}
+		else	//async execution
+		{
+			pthread_attr_t attr;
+			pthread_attr_init (&attr);
+			pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
+
+			gint res = pthread_create (&wthreadID, &attr, (gpointer)_e2_command_watch, rt);
+
+			pthread_attr_destroy (&attr);
+
+			gdk_threads_enter ();
+			e2_command_unblock_childsignal ();
+			if (res == 0)
+				return 0;
+		}
+	}
+	else //pid < 0, fork error
+	{
+		gdk_threads_enter ();
 		close (stdin_pipe[INWARDS]);
 		close (stdin_pipe[OUTWARDS]);
 		close (stdout_pipe[INWARDS]);
@@ -1014,107 +1145,6 @@ static gint _e2_command_fork (gchar *command, gchar **args, const gchar *cwd,
 		e2_command_unblock_childsignal ();
 		rt->status = E2_TASK_FAILED;
 		rt->ex.command.exit = 128;
-		goto launch_error;
-	}
-	//parent process
-	rt->status = E2_TASK_RUNNING;
-	rt->pid = (glong) pid;
-	rt->pidstr = g_strdup_printf ("%ld", rt->pid);
-	E2_CommandTaskData *cmddata = &rt->ex.command;
-	cmddata->child_stdin_fd = stdin_pipe[OUTWARDS];	//setup for later sending to child
-	cmddata->child_stdout_fd = stdout_pipe[INWARDS];	//setup for later receiving from child
-	cmddata->child_stderr_fd = stderr_pipe[INWARDS];	//setup for later receiving from child
-//	FILE *child_stdin_stream = fdopen (stdin_pipe[OUTWARDS], "w");
-//	FILE *child_stdout_stream = fdopen (stdout_pipe[INWARDS], "r");
-//	FILE *child_stderr_stream = fdopen (stderr_pipe[INWARDS], "r");
-//	cmddata->pollcount = 0;	//not ready to enable SIGCHILD yet NOT USED NOW
-	cmddata->sync = sync;
-	cmddata->extshell = extshell;
-	cmddata->show = show;
-#ifdef DEBUG_MESSAGES
-	if (sync)
-		printd (DEBUG, "synchronously run command %s", args[0]);
-	else
-		printd (DEBUG, "asynchronously run command %s", args[0]);
-#endif
-	if (show && e2_option_bool_get ("fileop-show"))
-	{
-		message = g_strconcat ((extshell) ? "sh>" : ">", command, NULL);
-		e2_output_print (rt->current_tab,
-			message, rt->pidstr, FALSE, "bold", "blue", NULL);
-		g_free (message);
-		message = g_strdup_printf ("  (%s)\n", rt->pidstr);
-		e2_output_print (rt->current_tab,
-			message, rt->pidstr, FALSE, "small", "blue", NULL);
-		g_free (message);
-		WAIT_FOR_EVENTS;
-	}
-
-//	e2_command_unblock_childsignal (); //allow child exit to be noticed SEE TIMER
-
-	close (stdin_pipe[INWARDS]);
-	close (stdout_pipe[OUTWARDS]);
-	close (stderr_pipe[OUTWARDS]);
-
-/* seems to do nothing
-	Setting the O_ASYNC flag for the read end of a pipe causes a signal
-	(SIGIO by default) to be generated when new input becomes available on the pipe.
-	On Linux, O_ASYNC is supported for pipes only since kernel 2.6.
-
-	//make pipes non-blocking, so we can read big buffers without having
-	//to use fionread() to prevent blocking
-	gint flags = fcntl (stdout_pipe[INWARDS], F_GETFL);
-	if (flags != -1)
-		fcntl (stdout_pipe[INWARDS], F_SETFL, flags | O_NONBLOCK | O_ASYNC);
-	flags = fcntl (stderr_pipe[INWARDS], F_GETFL);
-	if (flags != -1)
-		fcntl (stderr_pipe[INWARDS], F_SETFL, flags | O_NONBLOCK | O_ASYNC);
-*/
-/*	prefer to poll pipes at least once before unblocking SIGCHILD in this thread
-	(ostensibly it's not valid to unblock SIGCHILD in another thread where watching
-	happens, but for some kernels that does seem to help, so unblocking also happens
-	in _e2_command_watch())
-	So for async at least we use a timer which checks a flag set in _e2_command_watch()
-	CHECKME can timer persist past rt destruction at session end ? etc ?
-	Timer interval is compromise between prompt cleanup when there is little or no
-	initial input, and the amount of interruption of the initial input loop when
-	that involves a lot of printing - so we use a 2-stage timer
-*/
-	pthread_t wthreadID;
-	if (sync)
-	{
-		gdk_threads_leave ();	//BGL management happens downstream
-		if (pthread_create (&wthreadID, NULL, (gpointer)_e2_command_watch, rt))
-		{
-			e2_command_unblock_childsignal ();
-			gdk_threads_enter ();
-			goto launch_error;
-		}
-		//FIXME unblock-timer won't work until we go back to gtk
-//		usleep (10000);
-		e2_command_unblock_childsignal (); //allow any child's status change to be noticed
-		//FIXME allow command output to be printed while waiting for completion
-		//probably by running as for async, but blocking start of any other action
-		// or command (except send to child)
-		pthread_join (wthreadID, NULL);
-		gdk_threads_enter ();
-		return cmddata->exit;
-	}
-	else	//async execution
-	{
-//		g_timeout_add_full (G_PRIORITY_HIGH, 2,
-//			(GSourceFunc) _e2_command_checkblock, &cmddata->pollcount, NULL);
-		pthread_attr_t attr;
-		pthread_attr_init (&attr);
-		pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
-
-		gint res = pthread_create (&wthreadID, &attr, (gpointer)_e2_command_watch, rt);
-
-		pthread_attr_destroy (&attr);
-		e2_command_unblock_childsignal ();
-		if (res == 0)
-			return 0;
-		goto launch_error;
 	}
 
 launch_error:
@@ -1503,6 +1533,7 @@ static gint _e2_command_run_async (gchar *command, gchar **args, const gchar *cw
 		g_free (message);
 	}
 
+	usleep (100000);	//there may be some immediate messages to process before we want SIGCHILD
 	e2_command_unblock_childsignal ();
 
 	return pid;
@@ -1584,7 +1615,8 @@ static gint _e2_command_run_sync (gchar *command, gchar **args, const gchar *cwd
 	rt->status = (success) ? E2_TASK_COMPLETED : E2_TASK_FAILED;
 
 	//CHECKME revert CWD to curr_view->dir if that still exists ?
-
+	//FIXME this delay doesn't work properly
+	usleep (100000);	//there may be some immediate messages to process before we want SIGCHILD
 	e2_command_unblock_childsignal ();
 
 	rt->ex.command.exit = exit;
