@@ -1,4 +1,4 @@
-/* $Id: e2_dialog.c 1329 2008-10-22 02:26:53Z tpgww $
+/* $Id: e2_dialog.c 1365 2008-12-31 09:00:45Z tpgww $
 
 Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
@@ -39,17 +39,10 @@ ToDo - describe how dialogs work
 #endif
 #include <string.h>
 
-typedef struct _E2_Wait
-{
-	DialogButtons choice;
-	E2_MainLoop *loopdata;
-} E2_Wait;
-
 static DialogButtons _e2_dialog_line_input (gchar* window_title, gchar *prompt,
 	gchar *suggestion, gboolean select_text, //tag PASSWORDINPUT gboolean hide_text,
 	gchar **input, OW_ButtonFlags extras, gboolean history, GList **history_list);
 
-
 /**
 @brief remove dialog's bindings when it's being closed
 @param dialog the widget being closed
@@ -223,10 +216,6 @@ static void _e2_dialog_response_cb (GtkDialog *dialog, gint response,
 		e2_dialog_cancel_cb (NULL, GTK_WIDGET (dialog));
 }
 
-  /******************/
- /***** public *****/
-/******************/
-
 static void _e2_dialog_wait_response2_cb (GtkDialog *dialog, gint response, E2_Wait *w)
 {
 	w->choice = response;
@@ -246,13 +235,17 @@ static void _e2_dialog_wait_response_cb (GtkDialog *dialog, gint response, E2_Wa
 		w->choice = e2_dialog_response_decode (response);
 	e2_main_loop_quit (w->loopdata);
 }
+
+  /******************/
+ /***** public *****/
+/******************/
+
 /**
 @brief initiate a mainloop pending the user's choice in @a dialog
 This is instead of nested gtk_main();
 @param dialog the dialog where the response will be generated
 @param locked TRUE to open BGL while waiting
 @param maincontext TRUE to create mainloop in default context
-@param maincontext TRUE to use default maincontext for new loop
 @param multi TRUE when this dialog applies to multiple selected items
 @param getresult TRUE to decode the original response from @a dialog, FALSE just returns a default value (suitable to cancel any operation)
 @return enumerator corresponding to the user's choice
@@ -282,6 +275,11 @@ DialogButtons e2_dialog_wait (GtkWidget *dialog,
 		if (locked)
 			gdk_threads_enter ();
 //		pthread_cleanup_pop (0);
+		//prevent fatal multiple-callbacks if this func is repeatedly called
+		//on the same dialog e.g. inside a loop
+		g_signal_handlers_disconnect_matched (dialog,
+			G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+			0, 0, NULL, _e2_dialog_wait_response_cb, &wdata);
 	}
 	else
 		wdata.choice = (multi) ? NO_TO_ALL : CANCEL;
