@@ -1,4 +1,4 @@
-/* $Id: e2_mkdir_dialog.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2_mkdir_dialog.c 1363 2008-12-31 08:58:05Z tpgww $
 
 Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
@@ -989,7 +989,7 @@ static gboolean _e2_task_mkdirQ (E2_ActionTaskData *qed)
 		G_CALLBACK (_e2_mkdirdlg_changed_cb), rt);
 
 	*qed->status = E2_TASK_PAUSED;
-	e2_dialog_wait (rt->dialog, FALSE, FALSE, FALSE, TRUE);
+	e2_dialog_wait (rt->dialog, FALSE, FALSE, FALSE, FALSE);
 	*qed->status = E2_TASK_RUNNING;
 
 	_e2_mkdirdlg_dialog_destroy (rt);
