@@ -1,4 +1,4 @@
-/* $Id: e2_select_image_dialog.c 1197 2008-09-07 10:37:08Z tpgww $
+/* $Id: e2_select_image_dialog.c 1359 2008-12-05 03:04:09Z tpgww $
 
 Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 
@@ -208,7 +208,7 @@ static void _e2_sidlg_dir_change_cb (GtkFileChooser *chooser, E2_SID_Runtime *rt
 			data.spacedata = NULL;	//CHECKME ok?
 			_e2_sidlg_fill_custom_store (&data, rt);
 #else
-			_e2_sidlg_fill_custom_store (dirpath, rt);
+			_e2_sidlg_fill_custom_store ((VPATH*)dirpath, rt);
 #endif
 			//show the relevant icon, if any
 			_e2_sidlg_show_current (FALSE, rt);
