@@ -1,4 +1,4 @@
-/* $Id: e2_context_menu.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2_context_menu.c 1391 2009-01-18 20:45:24Z tpgww $
 
 Copyright (C) 2003-2008 tooar <tooar@gmx.net>
 Portions copyright (C) 1999 Michael Clark.
@@ -225,9 +225,9 @@ void e2_context_menu_add_filetype_item (GtkWidget *menu)
 		if (curr_view->spacedata != NULL)
 			exec = FALSE;//CHECKME commands for non-local items ?
 		else
-			exec = !e2_fs_access (&ddata, X_OK E2_ERR_NONE());
+			exec = e2_fs_is_exec2 (&ddata E2_ERR_NONE());
 #else
-		exec = !e2_fs_access (localpath, X_OK E2_ERR_NONE());
+		exec = e2_fs_is_exec2 (localpath E2_ERR_NONE());
 #endif
 		g_free (localpath);
 		freeext = FALSE;
