@@ -1,6 +1,6 @@
-/* $Id: e2_keybinding.c 1326 2008-10-21 20:53:53Z tpgww $
+/* $Id: e2_keybinding.c 1421 2009-02-09 22:25:29Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@gmx.net>
 
 This file is part of emelfm2.
 emelfm2 is free software; you can redistribute it and/or modify it
@@ -1154,11 +1154,32 @@ void e2_keybinding_output_help (gchar *parameters)
 					{
 						do
 						{
-							gtk_tree_model_get (mdl, &iter3, 1, &key, 3, &act, 4, &arg, -1);
-							g_string_append_printf (str, "\t\t%s\t%s\t%s\n", key, act, arg);
-							g_free (key);
-							g_free (act);
-							g_free (arg);
+							gtk_tree_model_get (mdl, &iter3, 0, &cat, 1, &key, 3, &act, 4, &arg, -1);
+							if (*cat != '\0')
+							{
+								GtkTreeIter iter4;
+								g_string_append_printf (str, "\t\t%s\n", cat);
+								g_free (cat);
+								if (gtk_tree_model_iter_children (mdl, &iter4, &iter3))
+								{
+									do
+									{
+										gtk_tree_model_get (mdl, &iter4, 1, &key, 3, &act, 4, &arg, -1);
+										g_string_append_printf (str, "\t\t\t%s\t%s\t%s\n", key, act, arg);
+										g_free (key);
+										g_free (act);
+										g_free (arg);
+									} while (gtk_tree_model_iter_next (mdl, &iter4));
+								}
+							}
+							else
+							{
+								g_string_append_printf (str, "\t\t%s\t%s\t%s\n", key, act, arg);
+								g_free (cat);
+								g_free (key);
+								g_free (act);
+								g_free (arg);
+							}
 						} while (gtk_tree_model_iter_next (mdl, &iter3));
 					}
 				}
@@ -1359,17 +1380,17 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t\t|<Shift>F12||",_A(14),".",_A(98),"|",NULL), //"panes.toggle direction"
 //	_I( single letters in the following not worth translating ?
 	//conform this help string to the following 'jump-keys'
-	g_strconcat("\t\t|<Control>","g","||",_A(10),".",_A(71),"|",_("Now press one of h,m,d\\n"),NULL), //"print"
+	g_strconcat("\t\t|<Control>g||",_A(10),".",_A(71),"|",_("Now press one of h,m,d\\n"),NULL), //"print"
 	//these 3 children are the so-called "directory jump keys"
-	g_strconcat("\t\t\t|","h","||cd|$HOME",NULL), //_A(20)
-	g_strconcat("\t\t\t|","m","||cd|/mnt",NULL),  //_A(20)
-	g_strconcat("\t\t\t|","d","||cd|$HOME/downloads",NULL),  //_A(20)
-	g_strconcat("\t\t|<Control>w","||",_A(10),".",_A(31),"|*,1",NULL),  //"adjust ratio" no arg translation
-	g_strconcat("\t\t|<Control>e","||",_A(10),".",_A(31),"|*,0",NULL),    //"adjust ratio" no arg translation
-	g_strconcat("\t\t|<Control>z","||",_A(1),".",_A(99),"|",NULL), //"focus toggle"
-	g_strconcat("\t\t|<Control>r","||",_A(14),".",_A(73),"|",NULL), //"refresh"
-	g_strconcat("\t\t|<Control>1","||",_A(5),".",_A(99),"|1",NULL), //"focus toggle" no arg translation
-	g_strconcat("\t\t|<Control>2","||",_A(5),".",_A(99),"|2",NULL), //"focus toggle"no arg translation
+	g_strconcat("\t\t\t|h||cd|$HOME",NULL), //_A(20)
+	g_strconcat("\t\t\t|m||cd|/mnt",NULL),  //_A(20)
+	g_strconcat("\t\t\t|d||cd|$HOME/downloads",NULL),  //_A(20)
+	g_strconcat("\t\t|<Control>w||",_A(10),".",_A(31),"|*,1",NULL),  //"adjust ratio" no arg translation
+	g_strconcat("\t\t|<Control>e||",_A(10),".",_A(31),"|*,0",NULL),    //"adjust ratio" no arg translation
+	g_strconcat("\t\t|<Control>z||",_A(1),".",_A(99),"|",NULL), //"focus toggle"
+	g_strconcat("\t\t|<Control>r||",_A(14),".",_A(73),"|",NULL), //"refresh"
+	g_strconcat("\t\t|<Control>1||",_A(5),".",_A(99),"|1",NULL), //"focus toggle" no arg translation
+	g_strconcat("\t\t|<Control>2||",_A(5),".",_A(99),"|2",NULL), //"focus toggle"no arg translation
 	g_strconcat("\t\t|<Control>Insert||",_A(1),".",_A(56),"|",NULL), //"insert selection"
 	g_strconcat("\t\t|<Alt>Insert||",_A(1),".",_A(56),"|",_A(116),NULL), //"insert selection, quote"
 	g_strconcat("\t\t",_C(33),"||||",NULL),  //_("panes"
@@ -1390,22 +1411,23 @@ static void _e2_keybinding_tree_defaults (E2_OptionSet *set)
 	g_strconcat("\t\t\t|Delete||",_A(6),".",_A(42),"|",NULL), //"delete"
 	g_strconcat("\t\t\t|<Shift>Delete||",_A(18),".",_A(42),"|",NULL), //"trash.delete"
 //	g_strconcat("\t\t\t|Home||cd|$HOME",NULL),   //_A(20)
-	g_strconcat("\t\t\t|<Control>a","||",_A(7),".",_A(100),"|",NULL), //"toggle select all"
-	g_strconcat("\t\t\t|<Control>i","||",_A(7),".",_A(57),"|",NULL), //"invert selection"
-	g_strconcat("\t\t\t|<Control>f","||",_A(6),".",_A(46),"|",NULL), //"find"
-	g_strconcat("\t\t\t|<Control>x","||",_A(14),".",_A(97),"|",NULL), //"sync"
-	g_strconcat("\t\t\t|<Control>h","||",_A(13),".",_A(83),"|",NULL),  //"toggle hidden"
-	g_strconcat("\t\t\t|<Control>m","||",_A(13),".",_A(24),"|",NULL), //"<bookmarks>"
-	g_strconcat("\t\t\t|<Control>t","||",_A(13),".",_A(25),"|",NULL), //"<filters>"
+	g_strconcat("\t\t\t|<Control>a||",_A(7),".",_A(100),"|",NULL), //"toggle select all"
+	g_strconcat("\t\t\t|<Control>i||",_A(7),".",_A(57),"|",NULL), //"invert selection"
+	g_strconcat("\t\t\t|<Control>d||",_A(7),".",_A(126),"|",NULL), //"select_type"
+	g_strconcat("\t\t\t|<Control>f||",_A(6),".",_A(46),"|",NULL), //"find"
+	g_strconcat("\t\t\t|<Control>x||",_A(14),".",_A(97),"|",NULL), //"sync"
+	g_strconcat("\t\t\t|<Control>h||",_A(13),".",_A(83),"|",NULL),  //"toggle hidden"
+	g_strconcat("\t\t\t|<Control>m||",_A(13),".",_A(24),"|",NULL), //"<bookmarks>"
+	g_strconcat("\t\t\t|<Control>t||",_A(13),".",_A(25),"|",NULL), //"<filters>"
 //	g_strconcat("\t\t\t|/||",_A(6),".",_A(46),"|",NULL),  //"find"
 	g_strconcat("\t\t\t|<Control>Left||",_A(14),".",_A(31),"|","*,1",NULL), //"adjust ratio" no arg translation
 	g_strconcat("\t\t\t|<Control>Right||",_A(14),".",_A(31),"|","*,0",NULL), //"adjust ratio" no arg translation
-	g_strconcat("\t\t\t|<Control>p","||",_A(13),".",_A(84),"|",NULL),  //"show menu"
-	g_strconcat("\t\t\t|<Shift><Control>p","||",_A(13),".",_A(84),"|",_A(117),NULL), //"show menu, shift"
-	g_strconcat("\t\t\t|<Control><Alt>p","||",_A(13),".",_A(84),"|",_A(109),NULL),  //"show menu, ctrl"
-	g_strconcat("\t\t\t|<Control><Alt>n","||",_A(7),".",_A(90),"|",NULL),  //"filelist.sortname"
-	g_strconcat("\t\t\t|<Control><Alt>m","||",_A(7),".",_A(89),"|",NULL),  //"filelist.sortmodified"
-	g_strconcat("\t\t\t|<Control><Alt>s","||",_A(7),".",_A(92),"|",NULL),  //"filelist.sortsize"
+	g_strconcat("\t\t\t|<Control>p||",_A(13),".",_A(84),"|",NULL),  //"show menu"
+	g_strconcat("\t\t\t|<Shift><Control>p||",_A(13),".",_A(84),"|",_A(117),NULL), //"show menu, shift"
+	g_strconcat("\t\t\t|<Control><Alt>p||",_A(13),".",_A(84),"|",_A(109),NULL),  //"show menu, ctrl"
+	g_strconcat("\t\t\t|<Control><Alt>n||",_A(7),".",_A(90),"|",NULL),  //"filelist.sortname"
+	g_strconcat("\t\t\t|<Control><Alt>m||",_A(7),".",_A(89),"|",NULL),  //"filelist.sortmodified"
+	g_strconcat("\t\t\t|<Control><Alt>s||",_A(7),".",_A(92),"|",NULL),  //"filelist.sortsize"
 	g_strconcat("\t\t",_C(12),"||||",NULL),  //_"directory line"
 	g_strconcat("\t\t\t|Tab||",_A(1),".",_A(35),"|",_A(110),NULL), //"complete dirs"
 	g_strconcat("\t\t\t|<Control>Tab||",_A(13),".",_A(64),"|",NULL), //"open"
