@@ -1,6 +1,6 @@
-/* $Id: e2_pane.c 1320 2008-10-13 08:55:21Z tpgww $
+/* $Id: e2_pane.c 1412 2009-02-01 21:51:22Z tpgww $
 
-Copyright (C) 2004-2008 tooar <tooar@gmx.net>
+Copyright (C) 2004-2009 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -332,10 +332,10 @@ E2_PaneRuntime *e2_pane_get_runtime (gpointer from, gpointer actiondata,
 }
 */
 /* *
-@brief change the directory for a specified pane and setup completion-watch
+@brief synchronously change the directory for a specified pane
 
-This is a wrapper for _e2_pane_change_dir (), with update of
-pane history list and hooklist
+This is a wrapper for _e2_pane_change_dir(), with update of pane history list
+and hooklist
 It must be called with gtk's thread-lock closed
 
 @param rt pointer to pane data struct, or NULL for current pane
@@ -345,14 +345,37 @@ It must be called with gtk's thread-lock closed
 @return
 */
 /*
-void e2_pane_change_dir_watch (E2_PaneRuntime *rt, gchar *path,
+void e2_pane_change_dir_sync (E2_PaneRuntime *rt, gchar *path,
 	E2_CDType *completed_flag)
 {
 	*completed_flag = CD_NOTFINISHED;
+	if (rt == NULL)
+		rt = curr_pane;
+	E2_Listman *data = (rt == &app.pane1) ?
+		&app.pane1_view.listcontrols : &app.pane2_view.listcontrols;
 	_e2_pane_change_dir (rt, path, completed_flag, TRUE, TRUE);
+	LISTS_LOCK
+	gchar *new = g_strdup (data->newpath); BAD, MAYBE NULL ALREADY
+	LISTS_UNLOCK
 	//FIXME wait for completion, without blocking the cd process
-//	app.timers[?] = g_timeout_add (100, (GSourceFunc) _e2_pane_cd_watch, &completed_flag);
+//	app.timers[?] = g_timeout_add (90, (GSourceFunc) _e2_pane_cd_watch, &completed_flag);
 	//FIXME nicely block while the timer is still running
+	while (TRUE)
+	{
+		gboolean wait;
+		LISTS_LOCK
+		wait = data->cd_requested || data->cd_working
+			|| !g_str_equal (new, rt->view->dir);
+		LISTS_UNLOCK
+		if (wait)
+			usleep (90000)
+		else
+		{
+			*completed_flag = CD_SUCCESS;
+			break;
+		}
+	}
+	g_free (new);
 }
 */
 /**
@@ -1229,6 +1252,73 @@ use a selection-change cb ? BGL management
 	return TRUE;
 }
 /**
+@brief select all items in specified pane which have same extension as first-selected item
+The pane is made active, if it wasn't already
+@param from the button, menu item etc which was activated
+@param art action runtime data
+
+@return TRUE if the pane is not empty
+*/
+static gboolean _e2_pane_select_extension (gpointer from, E2_ActionRuntime *art)
+{
+	E2_PaneRuntime *rt;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+
+	rt = e2_pane_get_runtime (from, art->data, NULL);
+	if (rt != curr_pane)
+		e2_pane_activate_other ();
+
+	while (TRUE)
+	{
+		LISTS_LOCK
+		//CHECKME abort if cd detected ?
+		gboolean busy = curr_view->listcontrols.refresh_working
+					 || curr_view->listcontrols.cd_working;
+		LISTS_UNLOCK
+		if (!busy)
+			break;
+		usleep (100000);
+	}
+
+	model = curr_view->model;
+	if (gtk_tree_model_get_iter_first (model, &iter));
+	{	//it's not empty
+		FileInfo *info = e2_fileview_get_selected_first_local (curr_view, FALSE);
+		if (info != NULL)
+		{
+			gboolean any;
+			gchar *base, *ext1;
+			GtkTreeSelection *sel = curr_view->selection;
+			gtk_tree_selection_unselect_all (sel);	//start with clean slate
+			any = e2_option_bool_get ("anycase-filetypes");
+			//get right-most extension
+			base = info->filename;
+			ext1 = strrchr (base, '.');
+			if (ext1 == base)
+				ext1 = NULL;	//ignore hidden-file indicator
+			do
+			{
+				gchar *ext2;
+				gtk_tree_model_get (model, &iter, FINFO, &info, -1);
+				//get extension
+				base = info->filename;
+				ext2 = strrchr (base, '.');
+				if (ext2 == base)
+					ext2 = NULL;	//ignore hidden-file indicator
+				if ((ext1 == NULL && ext2 == NULL)
+					|| (ext1 != NULL && ext2 != NULL
+						&& ((any && strcasecmp (ext1, ext2) == 0)
+						 || (!any && strcmp (ext1, ext2) == 0))))
+					gtk_tree_selection_select_iter (sel, &iter);
+			} while (gtk_tree_model_iter_next (model, &iter));
+
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+/**
 @brief invert selection state of all items in specified pane
 
 @param from the button, menu item etc which was activated
@@ -1620,6 +1710,9 @@ void e2_pane_actions_register (void)
 	action_name = g_strconcat(_A(7),".",_A(101),NULL);
  	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
 		_e2_pane_invert_current, NULL, TRUE);
+	action_name = g_strconcat(_A(7),".",_A(126),NULL);
+ 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
+		_e2_pane_select_extension, NULL, TRUE);
 	//CHECKME which exclusions for these sort-actions ?
 	action_name = g_strconcat(_A(7),".",_A(90),NULL);
 	e2_action_register (action_name, E2_ACTION_TYPE_ITEM,
