@@ -1,6 +1,6 @@
-/* $Id: e2_fs.c 1321 2008-10-13 09:17:38Z tpgww $
+/* $Id: e2_fs.c 1424 2009-02-10 22:11:01Z tpgww $
 
-Copyright (C) 2005-2008 tooar <tooar@gmx.net>
+Copyright (C) 2005-2009 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -329,8 +329,8 @@ gboolean e2_fs_ingroup (gid_t gid)
 }
 /**
 @brief execute the file(1) command on @a localpath, and search each line of its output for @a string
-
-Used in e2_fs_is_text and e2_fs_is_executable
+Used in e2_fs_is_text, e2_fs_is_exec2, and e2_fs_is_executable
+If @a locapath is a symlink, it will be traversed.
 
 @param localpath localised name of file to test
 @param string ascii string for which to search, in the output from file command
@@ -369,7 +369,7 @@ static gboolean _e2_fs_grep_file_output (VPATH *localpath, gchar *string E2_ERR_
 
 //tag E2_BADQUOTES
 		gchar *qp = e2_utils_quote_string (VPCSTR (localpath));
-		gchar *command = e2_utils_strcat ("file ", qp);	//want localised
+		gchar *command = e2_utils_strcat ("file -L ", qp);	//want localised
 		g_free (qp);
 		E2_FILE *pipe = e2_fs_open_pipe (command);
 		g_free (command);
@@ -442,12 +442,40 @@ gboolean e2_fs_is_text (VPATH *localpath E2_ERR_ARG())
 #endif
 }
 /**
+@brief test whether @a localpath is an executable item
+
+@param localpath localised name of item to test
+
+@return TRUE if it is an executable file (or at least, looks like one)
+*/
+gboolean e2_fs_is_exec2 (VPATH *localpath E2_ERR_ARG())
+{
+	if (e2_fs_access (localpath, X_OK E2_ERR_NONE()) == 0)
+	{
+#ifdef E2_VFS
+		if (e2_fs_item_is_mounted (localpath))
+			//FIXME "executable" doesn't grab all script files, but "application/" is no good
+			return _e2_fs_grep_file_output (localpath, "executable" E2_ERR_SAMEARG()); //do not translate
+		else	//item is virtual
+		{
+# ifdef E2_VFSTMP
+		//FIXME do this some other way for virtual items e.g. test mimetype
+# endif
+			return FALSE;
+		}
+#else
+		return _e2_fs_grep_file_output (localpath, "executable" E2_ERR_SAMEARG()); //do not translate
+#endif
+	}
+	return FALSE;
+}
+/**
 @brief test whether @a info belongs to an executable item
 If the item is a symlink, its target will be tested
 @param info pointer to a FileInfo data struct
 @param view pointer to view data struct from which to get path info for @a info
 
-@return TRUE if system reports x permission, and the file (1) command says it's executable
+@return TRUE if system reports x permission, and the file(1) command says it's an application file
 */
 gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view)
 {
@@ -470,6 +498,7 @@ gboolean e2_fs_is_executable (FileInfo *info, ViewInfo *view)
 			if (e2_fs_item_is_mounted (localpath))
 			{
 #endif
+				//see comment above about "executable" scan here
 				retval = _e2_fs_grep_file_output (localpath, "executable" E2_ERR_NONE ());  //string not to be translated
 #ifdef E2_VFS
 			}
@@ -722,7 +751,7 @@ gboolean e2_fs_cd_isok (gchar *utfpath E2_ERR_ARG())
 			success = FALSE;
 		}
 	}
-	else if (e2_fs_dir_is_native (utfpath))
+	else if (e2_fs_dir_is_native (utfpath, TRUE))
 		message = g_strdup_printf (_("Directory '%s' does not exist"), utfpath);
 	else
 	{
@@ -892,7 +921,7 @@ static gboolean _e2_fs_complete_dir (GtkWidget *entry, gchar *pressed_char, guin
 #ifdef E2_VFSTMP
 	//CHECKME enable vfs completion ??
 #endif
-	if (e2_fs_dir_is_native (utfdir))
+	if (e2_fs_dir_is_native (utfdir, FALSE))	//CHECKME parent-checking too?
 	{
 		GList *found = NULL;
 		retval = (e2_complete_str (&text, &pos, &found, E2_COMPLETE_FLAG_DIRS, pane) > 0);
@@ -931,6 +960,7 @@ gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane)
 	guint32 unikey = gdk_keyval_to_unicode (keyval);
 	if (unikey > 0)
 	{
+		gint start, end;
 		gchar keystring[8];
 		gint bytes = g_unichar_to_utf8 (unikey, keystring);
 		*(keystring+bytes) = '\0';
@@ -938,12 +968,14 @@ gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane)
 		switch (type)
 		{
 			case 1:	//insert completed text, and move after it
+				if (gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), &start, &end))
+					gtk_editable_delete_text (GTK_EDITABLE (entry), start, end);
 				if (_e2_fs_complete_dir (entry, keystring, pane))
 					return TRUE;
 				break;
 			case 2:	//select completed text
 			{
-				gint start, end, newpos, oldpos;
+				gint newpos, oldpos;
 				if (gtk_editable_get_selection_bounds (GTK_EDITABLE (entry), &start, &end))
 				{
 					//when there's a selection, cursor position = end, fix that
@@ -968,14 +1000,15 @@ gboolean e2_fs_complete_dir (GtkWidget *entry, guint keyval, guint pane)
 	return FALSE;
 }
 /**
-@brief evaluate whether @a utfpath is a native dir
+@brief evaluate whether @a utfpath is a native dir or optionally, a descendant of one
 Native means local and mounted, not fuse or other vfs
 This is for tailoring pre-change-dir and pre-refresh-dir checking
 @param utfpath absolute path of dir to check, utf-8 string
+@param descend TRUE to check for a descendant
 
 @return TRUE if @a utfpath is or seems to be native
 */
-gboolean e2_fs_dir_is_native (gchar *utfpath)
+gboolean e2_fs_dir_is_native (const gchar *utfpath, gboolean descend)
 {
 #if defined(__linux__) || defined(__FreeBSD__)
 	GList *fusemounts, *member;
@@ -996,9 +1029,9 @@ gboolean e2_fs_dir_is_native (gchar *utfpath)
 	gchar *localpath;
 	struct statfs fstatus;
 
-	localpath = F_FILENAME_TO_LOCALE (utfpath);
+	localpath = D_FILENAME_TO_LOCALE (utfpath);
+retry:
 	result = statfs (localpath, &fstatus);
-	F_FREE (localpath);
 	if (result)
 	{
 		if (result == ESTALE)
@@ -1006,31 +1039,41 @@ gboolean e2_fs_dir_is_native (gchar *utfpath)
 			printd (DEBUG, "%s resides in an unmounted VFS", utfpath);
 			//CHECKME mount it ?
 		}
+		g_free (localpath);
 		return TRUE;	//any operation on utfpath will fail ASAP
 	}
 	if (!(fstatus.f_flags & MNT_LOCAL))
+	{
+//		if (errno == EINTR)
+//			goto retry;
+		else if (errno == ENOENT && descend && e2_utils_get_parent_path (localpath, TRUE))
+			goto retry;
+		g_free (localpath);
 		return FALSE;
+	}
 #else
 	struct statvfs fstatus;
 	gchar *localpath;
-	localpath = F_FILENAME_TO_LOCALE (utfpath);
+	localpath = D_FILENAME_TO_LOCALE (utfpath);
 retry:
 	if (statvfs (localpath, &fstatus))
 	{
 		if (errno == EINTR)
 			goto retry;
-//		if (errno == ENOLINK)
+//		else if (errno == ENOLINK)
 //			re-mount it ??
-		F_FREE (localpath);
+		else if (errno == ENOENT && descend && e2_utils_get_parent_path (localpath, TRUE))
+			goto retry;
+		g_free (localpath);
 		return FALSE;
 	}
-	F_FREE (localpath);
 #endif
 	//FIXME MORE TESTS HERE e.g.
 	//from string contents e.g. "//:"
 	//from plugin data
 	//from vfs history data
 	//from other e2 data
+	g_free (localpath);
 	return TRUE;
 }
 #ifdef E2_VFS
@@ -2169,7 +2212,7 @@ gint e2_fs_put_stream (E2_FILE *stream, const gchar *string, gchar *localpath
 @param buffer start of memory to hold the read data
 @param bufsize size of @a buffer
 
-@return no. of bytes read, < 0 on error
+@return no. of bytes read, or -1 on error
 */
 ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize E2_ERR_ARG())
 {
@@ -2180,9 +2223,8 @@ ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize
 	do nread = read (descriptor, buffer, bufsize);
 		while (nread < 0 && E2_ERR_IS (EINTR));
 	return nread; */
-	if (bufsize == 0)
-		return 0;
-	ssize_t nread;
+
+	ssize_t nread = -1;	//error if buffer size is 0
 	size_t space = (size_t) bufsize;
 	size_t total = 0;
 	gpointer target = buffer;
@@ -2191,14 +2233,17 @@ ssize_t e2_fs_read (gint descriptor, gpointer buffer, /*size_t*/ gulong bufsize
 		nread = read (descriptor, target, space);
 		if (nread > 0)
 		{
-			space -= nread;
 			total += nread;
 			target += nread;	//irrelevant if we're finished now
+			space -= nread;
 		}
-		else if (nread == 0
-				|| errno != EINTR) //nread < 0
+		else if (nread == 0	//EOF, no error
+				||			//nread < 0
+				errno != EINTR) //un-recoverable error
 			break;
 	}
+	if (nread < 0)
+		return -1;
 	return (ssize_t)total;
 }
 /**
@@ -2211,12 +2256,12 @@ Error messages expect BGL open/off
 @param localpath path of file to be read, localised string
 @param contents store for ptr start of the allocated contents
 @param contlength store for no. of bytes written, or NULL if not interested
-@param string TRUE to add a trailing 0 if that's missing
+@param terminate TRUE to add a trailing 0 if that's not present in file as read
 
 @return TRUE if @a localpath was read successfully
 */
 gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
-	/*ssize_t*/ gulong *contlength, gboolean string E2_ERR_ARG())
+	/*ssize_t*/ gulong *contlength, gboolean terminate E2_ERR_ARG())
 {
 	gboolean retval;
 	const gchar *fmt;
@@ -2224,16 +2269,14 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 	if (e2_fs_item_is_mounted (localpath))
 	{
 #endif
-		struct stat sb;
-		retval = TRUE;
-		fmt = NULL;
 		gint fdesc = e2_fs_safeopen (VPCSTR(localpath), O_RDONLY, 0);
 		if (fdesc >= 0)
 		{
+			struct stat sb;
 			if (!e2_fs_fstat (fdesc, &sb))
 			{
 				size_t len = sb.st_size;
-				if (string)
+				if (terminate)
 					len += sizeof (gchar);	//in case we need to append a 0
 
 //				*contents = malloc (len);	//not size-limited, but imposes cleanup hassles
@@ -2249,7 +2292,9 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 					return FALSE;
 				}
 
-				/*ssize_t*/gulong nread = e2_fs_read (fdesc, *contents, sb.st_size E2_ERR_SAMEARG());
+				retval = TRUE;
+				fmt = NULL;
+				ssize_t nread = e2_fs_read (fdesc, *contents, sb.st_size E2_ERR_SAMEARG());
 				if (nread < 0 //|| nread < (gulong)sb.st_size
 				   )
 				{
@@ -2259,7 +2304,7 @@ gboolean e2_fs_get_file_contents (VPATH *localpath, gpointer *contents,
 					retval = FALSE;
 					fmt = _("Error reading file %s");
 				}
-				else if (string)
+				else if (terminate)
 				{
 					gchar *s;
 					if (nread >= sizeof (gchar))
@@ -2620,9 +2665,7 @@ Error message expects BGL to be closed
 */
 gboolean e2_fs_get_command_output (gchar *command, gpointer *output)
 {
-	gchar *local = F_FILENAME_TO_LOCALE (command);
-	E2_FILE *pipe = e2_fs_open_pipe (local);
-	F_FREE (local);
+	E2_FILE *pipe = e2_fs_open_pipe (command);
 	if (pipe == NULL)
 		return FALSE;
 	gchar *msg = g_strdup_printf (
@@ -2682,34 +2725,60 @@ gboolean e2_fs_get_command_output (gchar *command, gpointer *output)
 	return (total > 0);
 }
 /**
-@brief read up to @a count -1 bytes of piped command output into memory
-A terminating 0 is always added
+@brief read up to @a count -1 bytes of piped command output into newly-allocated memory
+A terminating 0 is appended after a successful read
 @param command localised command string to be executed
-@param contents ptr to where the contents are to be stored
+@param contents store for address of read contents
 @param count size of block to read + 1 (for added 0)
 
 @return TRUE if @a command result was read successfully
 */
 gboolean e2_fs_sniff_command_output (gchar *command, gpointer *contents, gulong count)
 {
-	gchar *local = F_FILENAME_TO_LOCALE (command);
-	E2_FILE *pipe = e2_fs_open_pipe (local);
-	F_FREE (local);
-	if (pipe == NULL)
-		return FALSE;
-	*contents = g_try_malloc (count);
-	if (*contents == NULL)
+	gboolean retval = FALSE;
+	E2_FILE *pipe = e2_fs_open_pipe (command);
+	if (pipe != NULL)
 	{
-		//FIXME warn user
+#ifdef USE_GLIB2_10
+		*contents = g_slice_alloc (count);
+#else
+		*contents = g_try_malloc (count);
+#endif
+		if (*contents != NULL)
+		{
+			ssize_t bytes_read, total = 0;
+reread:
+			errno = 0;
+			bytes_read = fread ((*contents + total), 1, (size_t)(count - total - 1), pipe);
+			if (bytes_read >= 0)
+			{
+				total += bytes_read;
+				if (total == count - 1) //got as much as we want
+					retval = TRUE;
+				else
+					goto reread;
+			}
+			else if (feof (pipe))
+			{
+				if (total > 0)
+					retval = TRUE;
+			}
+			else if (!ferror (pipe) || errno == EAGAIN || errno == EINTR)
+				goto reread;
+
+			if (retval)
+				*(*((gchar**)contents) + total) = '\0';
+			else
+				//FIXME warn user upon error
+#ifdef USE_GLIB2_10
+				g_slice_free1 (count, *contents);
+#else
+				g_free (*contents);
+#endif
+		}
 		e2_fs_pipe_close (pipe);
-		return FALSE;
 	}
-	ssize_t bytes_read = fread (*contents, 1, (size_t)count-1, pipe);
-	if (bytes_read < 0)
-		return FALSE;
-	*(*((gchar**)contents) + bytes_read) = '\0';
-	e2_fs_pipe_close (pipe);
-	return TRUE;
+	return retval;
 }
 /* *
 @brief blockwize read from stdin into memory
