@@ -1,6 +1,6 @@
-/* $Id: e2_hal.c 1343 2008-10-28 21:41:55Z tpgww $
+/* $Id: e2_hal.c 1427 2009-02-10 22:16:11Z tpgww $
 
-Copyright (C) 2008 tooar <tooar@gmx.net>
+Copyright (C) 2008-2009 tooar <tooar@gmx.net>
 
 This file is part of emelFM2.
 emelFM2 is free software; you can redistribute it and/or modify it
@@ -65,6 +65,7 @@ typedef struct _HalStorageData
 {
 	gchar *udi;	//for checking which devices are removed
 	gchar *device_path;
+//	gchar *mount_point;
 	guint timer_id;
 	gboolean ejectable;
 } HalStorageData;
@@ -89,6 +90,8 @@ static void _e2_hal_destroy_data (HalStorageData *data)
 		g_source_remove (data->timer_id);
 	if (data->udi != NULL)
 		g_free (data->udi);
+//	if (data->mount_point != NULL)
+//		g_free (data->mount_point);
 	if (data->device_path != NULL)
 		libhal_free_string (data->device_path);
 	DEALLOCATE (HalStorageData, data);
@@ -345,10 +348,8 @@ static gboolean _e2_hal_mount_volume (LibHalVolume *vol, const gchar *realdevice
 	str = g_strdup (mountpoint);
 	guint count = 0;
 #ifdef E2_VFS
-	VPATH ddata;
-	ddata.localpath = str;
-	ddata.spacedata = NULL;	//local mounting only
-	while (e2_fs_access3 (&ddata, F_OK) == 0)
+	VPATH ddata = { str, NULL };	//local mounting only
+	while (e2_fs_access3 (&ddata, F_OK, NULL) == 0)
 #else
 	while (e2_fs_access3 (str, F_OK) == 0)
 #endif
@@ -705,29 +706,34 @@ static gboolean _e2_hal_check_mount (HalStorageData *data)
 	LibHalDrive *drv = libhal_drive_from_device_file (ctx, data->device_path);
 	if (drv != NULL)
 	{
-		if (libhal_drive_is_media_detected (drv))
-		{
-			LibHalVolume *vol = libhal_volume_from_device_file (ctx, data->device_path);
-			if (vol != NULL)
-			{	//this device is itself a volume
-				if (!libhal_volume_is_mounted (vol))
+		LibHalVolume *vol = libhal_volume_from_device_file (ctx, data->device_path);
+		if (vol != NULL)
+		{	//this device is itself a volume
+			if (libhal_drive_is_media_detected (drv))
+			{
+				//mountpoint path needed for unmount processing
+//				data->mount_point = g_strdup (libhal_volume_get_mount_point (vol));
+				if(!libhal_volume_is_mounted (vol))
 				{
 					printd (DEBUG, "HAL: %s not mounted yet", data->device_path);
 					_e2_hal_mount_volume (vol, data->device_path);
 				}
-				libhal_volume_free (vol);
 			}
-			else
+			libhal_volume_free (vol);
+		}
+		else
+		{
+			gint volcount;
+			gchar **voludis = libhal_drive_find_all_volumes (ctx, drv, &volcount);
+			if (volcount > 0)
 			{
-				gint volcount;
-				gchar **voludis = libhal_drive_find_all_volumes (ctx, drv, &volcount);
-				if (volcount > 0)
+				gint i;
+				for (i = 0; i < volcount; i++)
 				{
-					gint i;
-					for (i = 0; i < volcount; i++)
+					vol = libhal_volume_from_udi (ctx, voludis[i]);
+					if (vol != NULL)
 					{
-						vol = libhal_volume_from_udi (ctx, voludis[i]);
-						if (vol != NULL)
+						if (libhal_drive_is_media_detected (drv))
 						{
 							if (libhal_volume_disc_is_blank (vol))
 							{
@@ -736,17 +742,21 @@ static gboolean _e2_hal_check_mount (HalStorageData *data)
 //								gdk_threads_leave ();
 								printd (WARN, "HAL: cannot mount blank medium");
 							}
-							else if (!libhal_volume_is_mounted (vol))
+							else
 							{
-								const gchar *realdevice = libhal_volume_get_device_file (vol);
-								printd (DEBUG, "HAL: %s not mounted yet", realdevice);
-								_e2_hal_mount_volume (vol, realdevice);
+//								data->mount_point = g_strdup (libhal_volume_get_mount_point (vol));
+								if (!libhal_volume_is_mounted (vol))
+								{
+									const gchar *realdevice = libhal_volume_get_device_file (vol);
+									printd (DEBUG, "HAL: %s not mounted yet", realdevice);
+									_e2_hal_mount_volume (vol, realdevice);
+								}
 							}
-							libhal_volume_free (vol);
 						}
+						libhal_volume_free (vol);
 					}
-					g_strfreev (voludis);
 				}
+				g_strfreev (voludis);
 			}
 		}
 		libhal_drive_free (drv);
@@ -893,6 +903,56 @@ static void _e2_hal_remove_device_cb (LibHalContext *ctx, const gchar *udi)
 			store = (HalStorageData *)node->data;
 			if (strcmp (store->udi, udi) == 0)
 			{
+/*THIS PROCESS IS SUPPOSED TO BE AUTOMATIC, WHEN A DISPLAYED DIR IS FOUND TO BE INACCESSIBLE
+				if (store->mount_point != NULL)
+				{
+					//make sure the unmounted dir is not shown, or blocking the unmount
+					gboolean found = FALSE;
+					gchar *elsewhere, *mountpoint, *s;
+					mountpoint = store->mount_point;
+					s = strrchr (mountpoint, G_DIR_SEPARATOR);	//assumes no trailer on mountpoint string
+					if (s > mountpoint + 1) //not doing root dir
+						elsewhere = g_strndup (mountpoint, s - mountpoint);
+					else //can't cd
+						elsewhere = g_strdup (G_DIR_SEPARATOR_S);
+#ifdef E2_VFSTMP
+		//FIXME dir when not mounted local
+#else
+					if (g_str_has_prefix (curr_view->dir, mountpoint))
+#endif
+					{
+						found = TRUE;
+						e2_fs_get_valid_path (&elsewhere, TRUE E2_ERR_NONE());
+						gdk_threads_enter ();	//in case of downstream error message
+						gchar *msg = g_strdup_printf (_("Cannot access %s, going to %s instead"),
+							curr_view->dir, elsewhere);
+#ifdef E2_VFSTMP
+						//FIXME cater for any space-change message
+#endif
+						e2_output_print_error (msg, TRUE);
+						gdk_threads_leave ();
+						e2_pane_change_dir (curr_pane, elsewhere);
+					}
+#ifdef E2_VFSTMP
+		//FIXME dir when not mounted local
+#else
+					if (g_str_has_prefix (other_view->dir, mountpoint))
+#endif
+					{
+						if (!found)
+							e2_fs_get_valid_path (&elsewhere, TRUE E2_ERR_NONE());
+						gchar *msg = g_strdup_printf (_("Cannot access %s, going to %s instead"),
+							other_view->dir, elsewhere);
+#ifdef E2_VFSTMP
+						//FIXME cater for any space-change message
+#endif
+						e2_output_print_error (msg, TRUE);
+						gdk_threads_leave ();
+						e2_pane_change_dir (other_pane, elsewhere);
+					}
+					g_free (elsewhere);
+				}
+*/
 				if (store->ejectable)
 				{
 					DBusError error;
