@@ -1,4 +1,4 @@
-/* $Id: e2_menu.c 1277 2008-09-27 01:10:48Z tpgww $
+/* $Id: e2_menu.c 1360 2008-12-12 19:00:56Z tpgww $
 
 Copyright (C) 2004-2008 tooar <tooar@gmx.net>
 
@@ -1014,7 +1014,7 @@ static void _e2_menu_tree_defaults (E2_OptionSet *set)
 	g_strdup("custom-menus=<"),  //internal name
 	g_strdup("fusemounts||||"),	//menu lookup name
 	//these are examples only, no translation
-	g_strconcat ("\t|vfs_on"E2ICONTB"|","_Mount","|","Mount tip","|","mount command $FTPDIR",NULL),
+	g_strconcat ("\t|vfs_on"E2ICONTB"|","_Mount","|","Mount tip","|","FTPDIR=/whatever;mount-command $FTPDIR",NULL),
 	g_strconcat ("\t|vfs_off"E2ICONTB"|","_Unmount choices","||",NULL),
 	g_strconcat ("\t\t||","_Unmount","|","Unount tip","|","fusermount -u $FTPDIR",NULL),
 	g_strdup(">"),
